@@ -13,6 +13,7 @@ typedef float  complex TCF;
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
 
 #define MACRO(B) do {B} while (0)
 #define ERROR(CODE) MACRO(return CODE;)
@@ -172,7 +173,7 @@ double vector_min(KDVEC(x)) {
 double vector_max_index(KDVEC(x)) {
     int k, r = 0;
     for (k = 1; k<xn; k++) {
-        if(xp[k]>xp[0]) {
+        if(xp[k]>xp[r]) {
             r = k;
         }
     }
@@ -182,7 +183,7 @@ double vector_max_index(KDVEC(x)) {
 double vector_min_index(KDVEC(x)) {
     int k, r = 0;
     for (k = 1; k<xn; k++) {
-        if(xp[k]<xp[0]) {
+        if(xp[k]<xp[r]) {
             r = k;
         }
     }
@@ -237,7 +238,7 @@ float vector_min_f(KFVEC(x)) {
 float vector_max_index_f(KFVEC(x)) {
     int k, r = 0;
     for (k = 1; k<xn; k++) {
-        if(xp[k]>xp[0]) {
+        if(xp[k]>xp[r]) {
             r = k;
         }
     }
@@ -247,7 +248,7 @@ float vector_max_index_f(KFVEC(x)) {
 float vector_min_index_f(KFVEC(x)) {
     int k, r = 0;
     for (k = 1; k<xn; k++) {
-        if(xp[k]<xp[0]) {
+        if(xp[k]<xp[r]) {
             r = k;
         }
     }
@@ -700,17 +701,24 @@ int saveMatrix(char * file, char * format, KDMAT(a)){
 
 ////////////////////////////////////////////////////////////////////////////////
 
-// http://c-faq.com/lib/gaussian.html
-double gaussrand()
+#ifdef __APPLE__
+
+#pragma message "randomVector is not thread-safe in OSX"
+
+inline double urandom() {
+    const long max_random = 2147483647; // 2**31 - 1
+    return (double)random() / (double)max_random;
+}
+
+double gaussrand(int *phase, double *pV1, double *pV2, double *pS)
 {
-	static double V1, V2, S;
-	static int phase = 0;
+	double V1=*pV1, V2=*pV2, S=*pS;
 	double X;
 
-	if(phase == 0) {
+	if(*phase == 0) {
 		do {
-			double U1 = (double)rand() / RAND_MAX;
-			double U2 = (double)rand() / RAND_MAX;
+            double U1 = urandom();
+			double U2 = urandom();
 
 			V1 = 2 * U1 - 1;
 			V2 = 2 * U2 - 1;
@@ -721,24 +729,30 @@ double gaussrand()
 	} else
 		X = V2 * sqrt(-2 * log(S) / S);
 
-	phase = 1 - phase;
+	*phase = 1 - *phase;
+    *pV1=V1; *pV2=V2; *pS=S;
 
 	return X;
+
 }
 
-int random_vector(int seed, int code, DVEC(r)) {
-    srand(seed);
+int random_vector(unsigned int seed, int code, DVEC(r)) {
+    int phase = 0;
+    double V1,V2,S;
+
+    srandom(seed);
+    
     int k;
     switch (code) {
       case 0: { // uniform
         for (k=0; k<rn; k++) {
-            rp[k] = (double)rand()/RAND_MAX;
+            rp[k] = urandom();
         }
         OK
       }
       case 1: { // gaussian
         for (k=0; k<rn; k++) {
-            rp[k] = gaussrand();
+            rp[k] = gaussrand(&phase,&V1,&V2,&S);
         }
         OK
       }
@@ -747,6 +761,77 @@ int random_vector(int seed, int code, DVEC(r)) {
     }
 }
 
+#else
+
+inline double urandom(struct random_data * buffer) {
+    int32_t res;
+    random_r(buffer,&res);
+    return (double)res/RAND_MAX;
+}
+
+
+// http://c-faq.com/lib/gaussian.html
+double gaussrand(struct random_data *buffer,
+                 int *phase, double *pV1, double *pV2, double *pS)
+{
+	double V1=*pV1, V2=*pV2, S=*pS;
+	double X;
+
+	if(*phase == 0) {
+		do {
+            double U1 = urandom(buffer);
+			double U2 = urandom(buffer);
+
+			V1 = 2 * U1 - 1;
+			V2 = 2 * U2 - 1;
+			S = V1 * V1 + V2 * V2;
+			} while(S >= 1 || S == 0);
+
+		X = V1 * sqrt(-2 * log(S) / S);
+	} else
+		X = V2 * sqrt(-2 * log(S) / S);
+
+	*phase = 1 - *phase;
+    *pV1=V1; *pV2=V2; *pS=S;
+
+	return X;
+
+}
+
+int random_vector(unsigned int seed, int code, DVEC(r)) {
+    struct random_data buffer;
+    char   random_state[128];
+    memset(&buffer, 0, sizeof(struct random_data));
+    memset(random_state, 0, sizeof(random_state));
+    
+    initstate_r(seed,random_state,sizeof(random_state),&buffer);
+    // setstate_r(random_state,&buffer);
+    // srandom_r(seed,&buffer);
+
+    int phase = 0;
+    double V1,V2,S;
+
+    int k;
+    switch (code) {
+      case 0: { // uniform
+        for (k=0; k<rn; k++) {
+            rp[k] = urandom(&buffer);
+        }
+        OK
+      }
+      case 1: { // gaussian
+        for (k=0; k<rn; k++) {
+            rp[k] = gaussrand(&buffer,&phase,&V1,&V2,&S);
+        }
+        OK
+      }
+
+      default: ERROR(BAD_CODE);
+    }
+}
+
+#endif
+
 ////////////////////////////////////////////////////////////////////////////////
 
 int smXv(KDVEC(vals),KIVEC(cols),KIVEC(rows),KDVEC(x),DVEC(r)) {
